from django.contrib.auth.models import User
from django.db.models import Model

from rules import predicate

from aleksis.core.models import Group, Person
from aleksis.core.util.predicates import has_global_perm, has_object_perm

from ..models import Room
from .chronos_helpers import get_classes, get_rooms, get_teachers


@predicate
def has_timetable_perm(user: User, obj: Model) -> bool:
    """Predicate which checks whether the user is allowed to access the requested timetable."""
    if type(obj) is Group:
        return (
            obj in user.person.member_of.all()
            or user.person.primary_group == obj
            or obj in user.person.owner_of.all()
            or has_global_perm("chronos.view_all_group_timetables")(user)
            or has_object_perm("core.view_group_timetable")(user, obj)
        )
    elif type(obj) is Person:
        return (
            user.person == obj
            or has_global_perm("chronos.view_all_person_timetables")(user)
            or has_object_perm("core.view_person_timetable")(user, obj)
        )
    elif type(obj) is Room:
        return has_global_perm("chronos.view_all_room_timetables")(user) or has_object_perm(
            "chronos.view_room_timetable"
        )(user, obj)
    else:
        return False


@predicate
def has_any_timetable_object(user: User) -> bool:
    """Predicate which checks whether there exists a timetable that the user is allowed to access."""
    return get_classes(user).exists() or get_rooms(user).exists() or get_teachers(user).exists()
